function Enew = CrankNicolson(dr, k0, Eold, Eoldold, n, zstep, AA, BB, epsilon, nref)  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Modification due to refractive index

MA = ones(n,1);
MB = ones(n,1);
MC = ones(n,1);
MD = zeros(n,1);

for j=2:n-1
  MA(j) = (epsilon(j-1) + epsilon(j))/(2.0*epsilon(j));
  MB(j) = (epsilon(j-1) + epsilon(j))/(4.0*epsilon(j-1)) + (epsilon(j+1) + epsilon(j))/(4.0*epsilon(j+1));
  MC(j) = (epsilon(j+1) + epsilon(j))/(2.0*epsilon(j));
end

idiag = 1i*k0*zstep/(4.0*nref);
for j=1:n
  MD(j) = idiag*( epsilon(j) - nref^2 );
end


%% End of modification auxiliaries
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A = ones(n,1);
B = -2.0*ones(n,1);
C = ones(n,1);
R = zeros(n,1);

idelta = 1i*zstep/(4.0*k0*nref*dr*dr);
bcoef  = zstep/(2*dr);

for row=1:n
A(row) =      -idelta*A(row)*AA(row)*MA(row) + 0.5*bcoef*BB(row);
B(row) = +1.0 -idelta*B(row)*AA(row)*MB(row) - MD(row);
C(row) =      -idelta*C(row)*AA(row)*MC(row) - 0.5*bcoef*BB(row);
end

LBNDRY = 0.0;
RBNDRY = 0.0;

%% Define the right-hand-side of the linear systsem to solve, using boundary ghost values:

row = 1;
  R(row) = Eold(row)*(1 + MD(row)) + idelta*(MA(row)*LBNDRY      -2.0*MB(row)*Eold(row) + MC(row)*Eold(row+1)  )*AA(row) + 0.5*bcoef*BB(row)*(Eold(row+1) - LBNDRY      ) ;


for row=2:n-1
  R(row) = Eold(row)*(1 + MD(row)) + idelta*(MA(row)*Eold(row-1) -2.0*MB(row)*Eold(row) + MC(row)*Eold(row+1)  )*AA(row) + 0.5*bcoef*BB(row)*(Eold(row+1) - Eold(row-1) ) ;
end

row = n; 
  R(row) = Eold(row)*(1 + MD(row)) + idelta*(MA(row)*Eold(row-1) -2.0*MB(row)*Eold(row) + MC(row)*RBNDRY       )*AA(row) + 0.5*bcoef*BB(row)*(RBNDRY      - Eold(row-1) ) ;



%% Solve the linear system L_minus E_new = L_plus E_old

Enew = TDMAsolver(A, B, C, R);

