%% Perfectly Matched Layer BC in BPM based on CN
%% Illustration of propagation in a leaky waveguide 

clear all; close all;

%% load input parameters
%%ParametersLeaky
ParametersLeaky

%% Definition of grid, AA, BB arrays
%%Grid
Grid

%% Permittivity spatial profile
eps  = ones(NX,1);
core = find(abs(cx) <= rho);
eps(core) = nco^2;
nref      = 1;

%% Initial condition = Gaussian 
%%w0 = 5.0e-06  ORIGINAL
w0 = 5.0e-06
E0 = GaussianBeam1D(cx,0.0,w0,k0,0.0,0.0);

%% or try an approximate leaky mode (fundamental)
%E0 = zeros(1,NX);
%E0(core) = cos(0.5*pi*cx(core)/rho);

%% this will hold the map of amplitude in z-x space
map = zeros(100,NX);
map(1,:) = E0;

Eold = E0;

%% PROPAGATE
tic;
for k=1:Nz

   Enew = CrankNicolson(dx, k0, Eold, Eold, NX, dz, AA, BB, eps, nref);  
   Eold = Enew;

%% Observe evolution = convergence to leaky fundamental
figure(3);
plot(abs(Eold).^2);
hold off;

   map(k,:) = Eold;
end
toc;

figure(1);
pcolor(log10(abs(map).^2 + 1.0e-20)); shading flat; colorbar;

figure(2);
plot(abs(Eold).^2);
hold on;
plot(abs(E0).^2,'r');
